use btleplug::{
    api::{Peripheral as _, PeripheralProperties},
    platform::{Peripheral, PeripheralId},
};
use serde::Serialize;

use crate::playlist::PlayMode;

#[derive(Debug, Clone, Default, Serialize)]
#[serde(rename_all = "camelCase")]
pub struct CentralState {
    pub is_ready: bool,
    pub is_scanning: bool,
    pub connected: Option<PeripheralId>,
}

#[derive(Debug, Clone, Serialize)]
#[serde(rename_all = "camelCase")]
pub struct PeripheralItem {
    pub id: PeripheralId,
    pub address: String,
    pub represent: String,
    pub is_connected: bool,
    pub rssi: Option<i16>,
    pub battery: Option<i16>,
}

#[derive(Debug, Clone, Default, Serialize)]
#[serde(rename_all = "camelCase")]
pub struct ChannelState {
    pub is_playing: bool,
    pub strength: u32,
    pub strength_limit: u32,
    pub is_boosting: bool,
    pub boost_level: u32,
    pub boost_limit: u32,
    pub play_mode: PlayMode,
}

impl PeripheralItem {
    pub async fn new(periperial: &Peripheral, properties: Option<&PeripheralProperties>) -> Self {
        Self {
            id: periperial.id(),
            address: periperial.address().to_string(),
            represent: properties
                .and_then(|p| p.local_name.clone())
                .unwrap_or(String::from("Unknown")),
            is_connected: periperial.is_connected().await.unwrap_or(false),
            rssi: properties.and_then(|p| p.rssi),
            battery: properties.and_then(|p| p.tx_power_level),
        }
    }
}
